import Flutter
import UIKit
import FawryPaySDK

public class SwiftFawrySdkPlugin: NSObject, FlutterPlugin {
    
    private static var eventSink : FlutterEventSink!
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        
        let channel = FlutterMethodChannel(name: Constants.sdk_channel_name, binaryMessenger: registrar.messenger())
        let instance = SwiftFawrySdkPlugin()
        
        registrar.addMethodCallDelegate(instance, channel: channel)
        let eventChannel = FlutterEventChannel(name: Constants.eventChannel_callback, binaryMessenger: registrar.messenger())
        eventChannel.setStreamHandler(FawryStreamHandler())
        
    }

    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
    let args = call.arguments as! [String:Any?]
    guard let fawryModelString = args[Constants.launch_model_json_arg] as? String else {
        handleInvalidArguments(message: Constants.Messages.invalidLaunchModel)
        return
    }

    guard let baseURL = args[Constants.baseURL_arg] as? String else {
        handleInvalidArguments(message: Constants.Messages.invalidBaseURL)
        return
    }

    guard let dartLang = args[Constants.language_arg] as? String else {
        handleInvalidArguments(message: Constants.Messages.invalidLanguage)
        return
    }

    let decoder = JSONDecoder()
    guard let jsonData = fawryModelString.data(using: .utf8) else {
        handleInvalidArguments(message: Constants.Messages.invalidLaunchModel)
        return
    }

    do {
        let fawryLaunchModel = try decoder.decode(FawryLaunchModel.self, from: jsonData)
        let lang = (dartLang == Constants.dart_arabic_lang ? (AppLanguage.Arabic) : (AppLanguage.English))

        switch call.method {
            case Constants.method_pay:
                startPayment(launchModel: fawryLaunchModel,baseURL: baseURL, lang: lang)
            case Constants.method_cards:
                showCardsManager(launchModel: fawryLaunchModel,baseURL: baseURL, lang: lang)
            default:
                startPayment(launchModel: fawryLaunchModel,baseURL: baseURL, lang: lang)
                break
        }

    } catch {
        handleDecodingError(error: error)
    }
}

    private func handleInvalidArguments(message: String) {
        let responseStatus = ResponseStatus(status: Constants.error_code, message: message, data: nil)
        SwiftFawrySdkPlugin.eventSink(responseStatus.convertToString)
    }

    private func handleDecodingError(error: Error) {
        let responseStatus = ResponseStatus(status: Constants.error_code, message: error.localizedDescription, data: nil)
        debugPrint(error)
        SwiftFawrySdkPlugin.eventSink(responseStatus.convertToString)
    }

    
    
    func startPayment (launchModel: FawryLaunchModel, baseURL: String, lang: String) {
        guard let fawryInstance = FrameworkHelper.shared else { return }
        guard let viewController = UIApplication.shared.delegate?.window??.rootViewController else { return }
        
        fawryInstance.launchAnonymousSDK(on: viewController,
                                         launchModel: launchModel,
                                         baseURL: baseURL,
                                         appLanguage: lang,
                                         enable3Ds: true,
                                         completionBlock: { (status) in
            guard let status = status else {return}
            let responseStatus = ResponseStatus(status: "\(status.rawValue)", message: Constants.Messages.paymentLoaded, data: nil)
            SwiftFawrySdkPlugin.eventSink(responseStatus.convertToString ?? "")
            
        }, onPreCompletionHandler: { (error) in
            let responseStatus = ResponseStatus(status: Constants.error_code, message: error?.message ?? "", data: nil)
            SwiftFawrySdkPlugin.eventSink(responseStatus.convertToString ?? "" )
        }, errorBlock: { (error) in
            let responseStatus = ResponseStatus(status: Constants.error_code , message: error?.message ?? "", data: nil)
            SwiftFawrySdkPlugin.eventSink(responseStatus.convertToString ?? "")
        }, onPaymentCompletedHandler: {
            (response) in
            
            if let chargeResponse = (response) as? PaymentChargeResponse {
                let responseStatus = ResponseStatus(status: Constants.payment_completed_code , message: Constants.Messages.paymentCompleted, data: chargeResponse.convertToString)
                SwiftFawrySdkPlugin.eventSink(responseStatus.convertToString ?? "")
            }
            
            if let errorResponse = response as? FawryError{
                let responseStatus = ResponseStatus(status: Constants.error_code , message: errorResponse.message ?? "", data: nil)
                SwiftFawrySdkPlugin.eventSink(responseStatus.convertToString ?? "" )
            }
            
        }, onSuccessHandler: { (response) in
            guard let responseCharge = response as? PaymentChargeResponse else {
                return}
            let responseStatus = ResponseStatus(status: Constants.success_code , message: Constants.Messages.paymentSucceeded, data: responseCharge.convertToString)
            SwiftFawrySdkPlugin.eventSink(responseStatus.convertToString ?? "" )
            
        })
    }
    
    func showCardsManager(launchModel: FawryLaunchModel, baseURL: String, lang: String) {
        guard let fawryInstance = FrameworkHelper.shared else { return }
        guard let viewController = UIApplication.shared.delegate?.window??.rootViewController else { return }

        fawryInstance.launchCardManager(on: viewController,
                                        launchModel: launchModel,
                                        baseURL: baseURL,
                                        appLanguage: lang,
                                        enable3Ds: true,
                                        errorBlock: { (error) in
                                            let responseStatus = ResponseStatus(status: Constants.error_code, message: error?.message ?? "", data: nil)
                                            SwiftFawrySdkPlugin.eventSink(responseStatus.convertToString ?? "")
                                        },
                                        onAddedNewCard: { card in
                                            // Code for handling a new card being added
                                        },
                                        dismissBlock: {
                                            // Code for handling dismissal
                                        })
    }

    class FawryStreamHandler: NSObject, FlutterStreamHandler {
        public func onListen(withArguments arguments: Any?, eventSink events: @escaping FlutterEventSink) -> FlutterError? {
            eventSink = events
            return nil
        }
        
        public func onCancel(withArguments arguments: Any?) -> FlutterError? {
            eventSink = nil
            return nil
        }
    }
    
}
